package cz.cvut.fit.swing.calipel.core;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public class BenchmarkSuite {
    private final List<String> benchmarks;
    private final Configuration configuration;

    public BenchmarkSuite(List<String> benchmarks, Configuration configuration) {
        this.benchmarks = benchmarks;
        this.configuration = configuration;
    }

    public List<Benchmark> getAllBenchmarks() {
        List<Benchmark> result = new ArrayList<Benchmark>();
        for (String benchmarkName : benchmarks) {
            result.add(lookupBenchmark(benchmarkName));
        }

        return result;
    }

    private Benchmark lookupBenchmark(String benchmarkName) {
        try {
            Class<?> benchmarkClass = getClass().getClassLoader().loadClass(
                  "cz.cvut.fit.swing.calipel.benchmarks." + benchmarkName);

            List<BenchmarkInstance> instances = new ArrayList<BenchmarkInstance>();
            for (Method method : benchmarkClass.getMethods()) {
                Object benchmarkImplementation = benchmarkClass.newInstance();
                String selector = method.getName();

                instances.add(new BenchmarkInstance(benchmarkImplementation, selector));
            }

            return new Benchmark(instances, configuration);
        } catch (Exception e) {
            throw new RuntimeException(
                  String.format("Don't know how to find [%s] benchmark", benchmarkName), e);
        }
    }
}
