#!/bin/sh

TOP=$(dirname $0)/../../../..

DEBUGGER=
CALLGRIND=
SMALLTALK_OPTS=
CALLGRIND_OPTS=
VM_OPTS=
VM=$TOP/stx/projects/smalltalk/stx

if [ ! -x "$VM" ]; then
	if [ -x "$TOP/../config.guess" ]; then
		VM=$TOP/stx/projects/smalltalk/build/$($TOP/../config.guess)/stx
	fi
fi


# On Windows use stx.com rather than stx.exe as exe has co
# console output

if [ ! -z "$OS" ]; then
    if [ "$OS" = "Windows_NT" ]; then
        VM=$VM.com
    fi
fi

if [ "$1" = "--help" ]
then
    "$VM" -I  --execute "$DIR/benchmark-runner.rc" ${1+"$@"}
# .........................
fi


while [ ! -z "$1" ]; do
    case "$1" in
        --gdb)
            DEBUGGER=gdb
            ;;
        --cgdb)
            DEBUGGER=cgdb
            ;;
        --callgrind)
            CALLGRIND="valgrind -v --smc-check=all-non-file --tool=callgrind --instr-atstart=no --collect-jumps=yes --cache-sim=yes --branch-sim=yes"
            ;;
        --stap)
            shift
            if [ -z "$1" ]; then 
                echo "--stap requires systemtap script to run"
                exit 1
            fi
            if [ ! -r "$1" ]; then
                echo "$1 is not a readable systemtap script"
                echo "--stap requires systemtap script to run"
                exit 1
            fi
            STAPSCRIPT=$1
            ;;        
        -T*)
            VM_OPTS="$VM_OPTS $1"
            ;;
        -M*)
            VM_OPTS="$VM_OPTS $1"
            ;;
        --abortOn*)
            VM_OPTS="$VM_OPTS $1"
            ;;
        --stopOn*)
	    VM_OPTS="$VM_OPTS $1"
	    ;;
        --immutableLiterals*)
	    VM_OPTS="$VM_OPTS $1"	    
	    ;;	    
        --noPrivacyCheck*)
	    VM_OPTS="$VM_OPTS $1"
	    ;;	    	    
        *)
            SMALLTALK_OPTS="$SMALLTALK_OPTS $1"
            ;;
    esac
    shift
done




DIR=`dirname $0`
if [ ! -z "$DEBUGGER" ]; then
    exec $DEBUGGER $VM
elif [ ! -z "$CALLGRIND" ]; then
    JITMAP=$(mktemp --suffix=.txt)
    $CALLGRIND $VM 				\
        $VM_OPTS 				\
        -I -Xdump-compile-map=$JITMAP 		\
        --execute "$DIR/benchmark-runner.rc"    \
        $SMALLTALK_OPTS
    echo "JIT map saved to: $JITMAP"
    exit $?
elif [ ! -z "$STAPSCRIPT" ]; then
    stap                                                                       \
        --unprivileged $STAPSCRIPT -I "$TOP/stx/hacking/tapset"                \
        -c "$VM $VM_OPTS --execute $DIR/benchmark-runner.rc $SMALLTALK_OPTS"
    exit $?
else
    exec "$VM" \
        $VM_OPTS 					\
        -I  --execute "$DIR/benchmark-runner.rc" 	\
        $SMALLTALK_OPTS
fi
