class ResultsFilter
  include ActiveModel::Model

  attr_accessor :benchmark_configurations, :tags

  def initialize(*args)
    super

    @benchmark_configurations ||= {}
    @tags ||= {}
  end

  def benchmark_configurations_relation
    BenchmarkConfiguration.where(id: benchmark_configurations.compact)
  end

  # Return benchmark reports (batches in webapp) based on criterie. 
  # The filtering works as folloeing: 
  #  * if selected configurations and selected benchmarks are BOTH ARE EMPTYy, 
  #    return an empty list. 
  #  * if selected CONFIGURATIONS ARE EMPTY, then filter by selected tags.
  #  * if selected TAGS ARE EMPTY, then filter by selected configrations
  #  * if BOTH ARE NON-EMPTY, then return only those having one of the
  #    selected configurations and one of the tags
  def filtered_batches
    benchmark_configurations.compact! 
    benchmark_configurations.reject! { | e | e.empty? }
    tags.compact!
    tags.reject! { | e | e.empty? }

    if benchmark_configurations.empty? and tags.empty? then
      return []
    end
    batches = BenchmarkBatch.includes(
                              :tags,
                              :benchmark_configuration,
                              benchmark_results: [
                                :benchmark_durations,
                                :benchmark_info
                              ])
    if not benchmark_configurations.empty? then
      batches = batches.where(benchmark_configuration_id: benchmark_configurations)
    end
    if not tags.empty? then
      batches = batches.joins(:tags).where(tags: { id: tags})
    end
    batches = batches.order("performed_at desc")
    return batches
  end

end
